program burgers

  use fourier
  use deriv

  implicit none
  integer, parameter :: N = 128
  double precision, dimension(N) :: x, u, du
  double precision, dimension(N/2) :: ur, ui, dur, dui, d2ur, d2ui
  integer :: i, k, N2, nt, Nend
  double precision :: pi, nu = 1.d-2, dx, time, dt = 1.d-3, Tend = 10.d0

  pi = dacos( -1.d0 )
  dx = 2.d0 * pi / N
  N2 = N / 2
  Nend = nint( Tend / dt )
  time = 0.d0

  do i = 1, N
     x(i) = (i-1) * dx
     u(i) = sin( x(i) )
  enddo

  call initfft(N)

  call fft( N, u, ur, ui, 1 )
  print *,'set title "t =', time, '"'
  print *,'set xlabel "x"'
  print *,'set ylabel "u"'
  print *,'set xrange [0:2*pi]'
  print *,'set yrange [-1:+1]'
  print *,'plot "-" using 1:2 with lines'
  do i = 1, N
     print *, x(i), u(i)
  enddo
  print *,'e'

  do nt = 1, Nend

     time = nt * dt

     ! Calcolo derivata prima e seconda
     call derx( N, ur, ui, dur, dui )
     call derxx( N, ur, ui, d2ur, d2ui )
     
     ! Calcolo del termine non-lineare
     call fft( N, du, dur, dui, -1 )
     do i = 1, N
        du(i) = u(i) * du(i)
     enddo
     call fft( N, du, dur, dui, +1 )
     
     ! Schema di Eulero forward
     do k = 1, N2
        ur(k) = ur(k) + dt * ( -dur(k) + nu * d2ur(k) )
        ui(k) = ui(k) + dt * ( -dui(k) + nu * d2ui(k) )
     enddo
     call fft( N, u, ur, ui, -1 )
     
     if ( mod( nt, 10 ) == 0 ) then
        ! Stampa del risultato in formato gnuplot
        print *,'set title "t =', time, '"'
        print *,'set xlabel "x"'
        print *,'set ylabel "u"'
        print *,'set xrange [0:2*pi]'
        print *,'set yrange [-1:+1]'
        print *,'plot "-" using 1:2 with lines'
        do i = 1, N
           print *, x(i), u(i)
        enddo
        print *,'e'
     endif

  enddo

  call endfft

end program burgers
