module fourier

  public initfft, endfft, fft
  private realft, four1

  double precision, allocatable :: fftwork(:)

contains

! Subroutine initfft: inizializza vettori di lavoro per fft

subroutine initfft(N)

  integer, intent(IN) :: N

  allocate(fftwork(N))

end subroutine initfft


! Subroutine endfft: disalloca vettori di lavoro per fft

subroutine endfft()

  deallocate(fftwork)

end subroutine endfft


! Subroutine fft: esegue la trasformata diretta o inversa

subroutine fft(N, f, ar, ai, isign)

  implicit none
  integer, intent(IN) :: N, isign
  double precision, dimension(N) :: f
  double precision, dimension(N/2) :: ar, ai
  integer :: i,k,n2

  n2=N/2
  if ( isign == +1 ) then
     ! Trasformata diretta
     do i=1,N
        fftwork(i)=f(i)
     enddo
     call realft(fftwork,N,isign)
     i=1
     do k=1,n2
        ar(k)=fftwork(i)/N
        i = i+1
        ai(k)=fftwork(i)/N
        i = i+1
     enddo
     ai(1)=0.d0
  else
     ! Trasformata inversa
     i=1
     do k=1,n2
        fftwork(i)=ar(k)
        i = i+1
        fftwork(i)=ai(k)
        i = i+1
     enddo
     fftwork(2)=0.d0
     call realft(fftwork,N,isign)
     do i=1,N
        f(i)=2.d0*fftwork(i)
     enddo
  endif

end subroutine fft


! Subroutines del numerical recipes per FFT

  SUBROUTINE realft(data,n,isign)
    IMPLICIT NONE
    
    INTEGER :: isign,n
    DOUBLE PRECISION :: data(n)
!   USES four1
    INTEGER :: i,i1,i2,i3,i4,n2p3
    DOUBLE PRECISION :: c1,c2,h1i,h1r,h2i,h2r,wis,wrs
    DOUBLE PRECISION :: theta,wi,wpi,wpr,wr,wtemp

    theta=3.141592653589793d0/dble(n/2)
    c1=0.5d0
    IF (isign == 1) THEN
       c2=-0.5d0
       CALL four1(data,n/2,+1)
    ELSE
       c2=0.5d0
       theta=-theta
    ENDIF
    wpr=-2.0d0*sin(0.5d0*theta)**2
    wpi=sin(theta)
    wr=1.0d0+wpr
    wi=wpi
    n2p3=n+3
    DO i=2,n/4
       i1=2*i-1
       i2=i1+1
       i3=n2p3-i2
       i4=i3+1
       wrs=dble(wr)
       wis=dble(wi)
       h1r=c1*(data(i1)+data(i3))
       h1i=c1*(data(i2)-data(i4))
       h2r=-c2*(data(i2)+data(i4))
       h2i=c2*(data(i1)-data(i3))
       data(i1)=h1r+wrs*h2r-wis*h2i
       data(i2)=h1i+wrs*h2i+wis*h2r
       data(i3)=h1r-wrs*h2r+wis*h2i
       data(i4)=-h1i+wrs*h2i+wis*h2r
       wtemp=wr
       wr=wr*wpr-wi*wpi+wr
       wi=wi*wpr+wtemp*wpi+wi
    ENDDO
    IF (isign == 1) THEN
       h1r=data(1)
       data(1)=h1r+data(2)
       data(2)=h1r-data(2)
    ELSE
       h1r=data(1)
       data(1)=c1*(h1r+data(2))
       data(2)=c1*(h1r-data(2))
       CALL four1(data,n/2,-1)
    ENDIF
  END SUBROUTINE realft




  SUBROUTINE four1(data,nn,isign)
    IMPLICIT NONE

    INTEGER :: isign,nn
    DOUBLE PRECISION :: data(2*nn)
    INTEGER :: i,istep,j,m,mmax,n
    DOUBLE PRECISION :: tempi,tempr
    DOUBLE PRECISION :: theta,wi,wpi,wpr,wr,wtemp
    
    n=2*nn
    j=1
    DO i=1,n,2
       IF (j > i) THEN
          tempr=data(j)
          tempi=data(j+1)
          data(j)=data(i)
          data(j+1)=data(i+1)
          data(i)=tempr
          data(i+1)=tempi
       ENDIF
       m=n/2
       DO WHILE ((m >= 2).AND.(j > m))
          j=j-m
          m=m/2
       ENDDO
       j=j+m
    ENDDO
    mmax=2
    DO WHILE (n.gt.mmax)
       istep=2*mmax
       theta=6.28318530717959d0/(isign*mmax)
       wpr=-2.d0*sin(0.5d0*theta)**2
       wpi=sin(theta)
       wr=1.d0
       wi=0.d0
       DO m=1,mmax,2
          DO i=m,n,istep
             j=i+mmax
             tempr=dble(wr)*data(j)-dble(wi)*data(j+1)
             tempi=dble(wr)*data(j+1)+dble(wi)*data(j)
             data(j)=data(i)-tempr
             data(j+1)=data(i+1)-tempi
             data(i)=data(i)+tempr
             data(i+1)=data(i+1)+tempi
          ENDDO
          wtemp=wr
          wr=wr*wpr-wi*wpi+wr
          wi=wi*wpr+wtemp*wpi+wi
       ENDDO
       mmax=istep
    ENDDO
  END SUBROUTINE four1

end module fourier
